@@ -15,7 +15,7 @@
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  *
- *  $Id: print-dialog.c,v 1.9 2003/08/15 10:03:27 chpe Exp $
+ *  $Id: print-dialog.c,v 1.9.2.1 2003/09/29 18:22:58 chpe Exp $
  */
 
 #include "print-dialog.h"
@@ -288,6 +288,7 @@ print_get_info (EphyDialog *dialog)
 	GValue page_url = {0, };
 	GValue date = {0, };
 	GValue page_numbers = {0, };
+	const char *filename;
 
 	info = g_new0 (EmbedPrintInfo, 1);
 
@@ -300,7 +301,15 @@ print_get_info (EphyDialog *dialog)
 	g_value_unset (&printer);
 
 	ephy_dialog_get_value (dialog, FILE_PROP, &file);
-	info->file = gnome_vfs_expand_initial_tilde (g_value_get_string (&file));
+	filename = g_value_get_string (&file);
+	if (filename != NULL)
+	{
+		info->file = gnome_vfs_expand_initial_tilde (g_value_get_string (&file));
+	}
+	else
+	{
+		info->file = NULL;
+	}
 	g_value_unset (&file);
 
 	ephy_dialog_get_value (dialog, BOTTOM_PROP, &bottom_margin);
