@@ -15,10 +15,14 @@
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  *
- *  $Id: ephy-node.c,v 1.18 2003/05/27 09:57:51 xan Exp $
+ *  $Id: ephy-node.c,v 1.25 2003/07/13 09:23:57 chpe Exp $
  */
 
+#ifdef HAVE_CONFIG_H
 #include <config.h>
+#endif
+
+#include <glib.h>
 #include <bonobo/bonobo-i18n.h>
 #include <stdlib.h>
 #include <string.h>
@@ -64,6 +68,12 @@ struct EphyNode
 	EphyNodeDb *db;
 };
 
+typedef struct
+{
+	EphyNodeSignalType type;
+	va_list valist;
+} ENESCData;
+
 /* evillish hacks to temporarily readlock->writelock and v.v. */
 static inline void
 write_lock_to_read_lock (EphyNode *node)
@@ -113,12 +123,16 @@ int_hash (gconstpointer a)
 }
 
 static void
-callback (long id, EphyNodeSignalData *data, gpointer *user_data)
+callback (long id, EphyNodeSignalData *data, gpointer *dummy)
 {
-	va_list valist = (va_list) user_data[0];
-	EphyNodeSignalType type = GPOINTER_TO_INT (user_data[1]);
+	ENESCData *user_data;
+	va_list valist;
 
-	if (data->type != type) return;
+	user_data = (ENESCData *) dummy;
+
+	G_VA_COPY(valist, user_data->valist);
+
+	if (data->type != user_data->type) return;
 
 	switch (data->type)
 	{
@@ -148,23 +162,24 @@ callback (long id, EphyNodeSignalData *data, gpointer *user_data)
 			data->callback (data->node, va_arg (valist, int *), data->data);
 		break;
 	}
+
+        va_end(valist);
 }
 
 static void
 ephy_node_emit_signal (EphyNode *node, EphyNodeSignalType type, ...)
 {
-	va_list valist;
-	gpointer data[2];
+	ENESCData data;
 
-	va_start (valist, type);
+	va_start (data.valist, type);
+
+	data.type = type;
 
-	data[0] = (gpointer)valist;
-	data[1] = GINT_TO_POINTER (type);
 	g_hash_table_foreach (node->signals,
 			      (GHFunc) callback,
-			      data);
+			      &data);
 
-	va_end (valist);
+	va_end (data.valist);
 }
 
 static void
@@ -185,13 +200,14 @@ ephy_node_finalize (EphyNode *node)
 			g_free (val);
 		}
 	}
-	g_ptr_array_free (node->properties, FALSE);
+	g_ptr_array_free (node->properties, TRUE);
 
 	g_hash_table_destroy (node->parents);
 
-	g_ptr_array_free (node->children, FALSE);
+	g_ptr_array_free (node->children, TRUE);
 
 	g_static_rw_lock_free (node->lock);
+	g_free (node->lock);
 
 	g_free (node);
 }
@@ -271,9 +287,12 @@ unref_signal_objects (long id,
 	              EphyNodeSignalData *signal_data,
 	              EphyNode *node)
 {
-	g_object_weak_unref (G_OBJECT (signal_data->data),
-			     (GWeakNotify)signal_object_weak_notify,
-			     signal_data);
+	if (signal_data->data)
+	{
+		g_object_weak_unref (G_OBJECT (signal_data->data),
+				     (GWeakNotify)signal_object_weak_notify,
+				     signal_data);
+	}
 }
 
 static void
@@ -767,6 +786,7 @@ ephy_node_save_to_xml (EphyNode *node,
 {
 	xmlNodePtr xml_node;
 	char *xml;
+	char xml_buf [G_ASCII_DTOSTR_BUF_SIZE];
 	guint i;
 
 	g_return_if_fail (EPHY_IS_NODE (node));
@@ -820,14 +840,14 @@ ephy_node_save_to_xml (EphyNode *node,
 			g_free (xml);
 			break;
 		case G_TYPE_FLOAT:
-			xml = g_strdup_printf ("%f", g_value_get_float (value));
-			xmlNodeSetContent (value_xml_node, xml);
-			g_free (xml);
+			g_ascii_dtostr (xml_buf, sizeof (xml_buf), 
+					g_value_get_float (value));
+			xmlNodeSetContent (value_xml_node, xml_buf);
 			break;
 		case G_TYPE_DOUBLE:
-			xml = g_strdup_printf ("%f", g_value_get_double (value));
-			xmlNodeSetContent (value_xml_node, xml);
-			g_free (xml);
+			g_ascii_dtostr (xml_buf, sizeof (xml_buf),
+					g_value_get_double (value));
+			xmlNodeSetContent (value_xml_node, xml_buf);
 			break;
 		case G_TYPE_POINTER:
 		{
@@ -953,10 +973,10 @@ ephy_node_new_from_xml (EphyNodeDb *db, xmlNodePtr xml_node)
 				g_value_set_long (value, atol (xml));
 				break;
 			case G_TYPE_FLOAT:
-				g_value_set_float (value, atof (xml));
+				g_value_set_float (value, g_ascii_strtod (xml, NULL));
 				break;
 			case G_TYPE_DOUBLE:
-				g_value_set_double (value, atof (xml));
+				g_value_set_double (value, g_ascii_strtod (xml, NULL));
 				break;
 			case G_TYPE_POINTER:
 			{
@@ -1308,7 +1328,7 @@ ephy_node_signal_connect_object (EphyNode *node,
 	int ret;
 
 	g_return_val_if_fail (EPHY_IS_NODE (node), -1);
-	
+
 	signal_data = g_new0 (EphyNodeSignalData, 1);
 	signal_data->node = node;
 	signal_data->id = node->signal_id;
@@ -1319,8 +1339,13 @@ ephy_node_signal_connect_object (EphyNode *node,
 	g_hash_table_insert (node->signals,
 			     GINT_TO_POINTER (node->signal_id),
 			     signal_data);
-	g_object_weak_ref (object, (GWeakNotify)signal_object_weak_notify,
-			   signal_data);
+	if (object)
+	{
+		g_object_weak_ref (object,
+				   (GWeakNotify)signal_object_weak_notify,
+				   signal_data);
+	}
+
 	ret = node->signal_id;
 	node->signal_id++;
 
