@@ -1,5 +1,6 @@
 /*
- *  Copyright (C) 2002  Ricardo Fernndez Pascual
+ *  Copyright (C) 2002  Ricardo Fernández Pascual
+ *  Copyright (C) 2003  Christian Persch
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -19,15 +20,17 @@
 #include "ephy-location-entry.h"
 #include "ephy-autocompletion-window.h"
 #include "ephy-marshal.h"
-#include "ephy-gobject-misc.h"
 #include "eel-gconf-extensions.h"
 #include "ephy-prefs.h"
 #include "ephy-debug.h"
 
+#include <glib-object.h>
+#include <gtk/gtkhbox.h>
 #include <gtk/gtkentry.h>
 #include <gtk/gtkwindow.h>
 #include <gdk/gdkkeysyms.h>
 #include <gtk/gtkmain.h>
+#include <gtk/gtktooltips.h>
 #include <libgnomeui/gnome-entry.h>
 #include <string.h>
 
@@ -37,18 +40,16 @@
 struct _EphyLocationEntryPrivate {
 	GtkWidget *combo;
 	GtkWidget *entry;
-	gchar *before_completion;
+	char *before_completion;
 	EphyAutocompletion *autocompletion;
 	EphyAutocompletionWindow *autocompletion_window;
 	gboolean autocompletion_window_visible;
-	gint autocompletion_timeout;
 	gint show_alternatives_timeout;
 	gboolean block_set_autocompletion_key;
 	gboolean going_to_site;
-	gboolean editing;
+	gboolean user_changed;
 
-	gchar *autocompletion_key;
-	gchar *last_completion;
+	char *autocompletion_key;
 	char *last_action_target;
 };
 
@@ -58,37 +59,47 @@ struct _EphyLocationEntryPrivate {
 /**
  * Private functions, only availble from this file
  */
-static void		ephy_location_entry_class_init		(EphyLocationEntryClass *klass);
-static void		ephy_location_entry_init		(EphyLocationEntry *w);
-static void		ephy_location_entry_finalize_impl	(GObject *o);
-static void		ephy_location_entry_build		(EphyLocationEntry *w);
-static gboolean		ephy_location_entry_key_press_event_cb  (GtkWidget *entry, GdkEventKey *event,
-								 EphyLocationEntry *w);
-static void		ephy_location_entry_activate_cb		(GtkEntry *entry,
-								 EphyLocationEntry *w);
-static void		ephy_location_entry_autocompletion_sources_changed_cb (EphyAutocompletion *aw,
-									       EphyLocationEntry *w);
-static gint		ephy_location_entry_autocompletion_show_alternatives_to (EphyLocationEntry *w);
-static void		ephy_location_entry_autocompletion_window_url_activated_cb
-/***/								(EphyAutocompletionWindow *aw,
-								 const gchar *target,
-								 int action,
-								 EphyLocationEntry *w);
-static void		ephy_location_entry_list_event_after_cb (GtkWidget *list,
-								 GdkEvent *event,
-								 EphyLocationEntry *e);
-static void		ephy_location_entry_editable_changed_cb (GtkEditable *editable,
-								 EphyLocationEntry *e);
-static void		ephy_location_entry_set_autocompletion_key (EphyLocationEntry *e);
-static void		ephy_location_entry_autocompletion_show_alternatives (EphyLocationEntry *w);
-static void		ephy_location_entry_autocompletion_hide_alternatives (EphyLocationEntry *w);
-static void		ephy_location_entry_autocompletion_window_hidden_cb (EphyAutocompletionWindow *aw,
-									     EphyLocationEntry *w);
+static void	ephy_location_entry_class_init		(EphyLocationEntryClass *klass);
+static void	ephy_location_entry_init		(EphyLocationEntry *w);
+static void	ephy_location_entry_finalize_impl	(GObject *o);
+static void	ephy_location_entry_construct_contents	(EphyLocationEntry *w);
+static gboolean	ephy_location_entry_key_press_event_cb  (GtkWidget *entry, GdkEventKey *event,
+							 EphyLocationEntry *w);
+static void	ephy_location_entry_activate_cb		(GtkEntry *entry,
+							 EphyLocationEntry *w);
+static void	ephy_location_entry_autocompletion_sources_changed_cb
+							(EphyAutocompletion *aw,
+							 EphyLocationEntry *w);
+static gint	ephy_location_entry_autocompletion_show_alternatives_to (EphyLocationEntry *w);
+static void	ephy_location_entry_autocompletion_window_url_activated_cb
+							(EphyAutocompletionWindow *aw,
+							 const gchar *target,
+							 int action,
+							 EphyLocationEntry *w);
+static void	ephy_location_entry_list_event_after_cb (GtkWidget *list,
+							 GdkEvent *event,
+							 EphyLocationEntry *e);
+static void	ephy_location_entry_editable_changed_cb (GtkEditable *editable,
+							 EphyLocationEntry *e);
+static void	ephy_location_entry_set_autocompletion_key (EphyLocationEntry *e);
+static void	ephy_location_entry_autocompletion_show_alternatives (EphyLocationEntry *w);
+static void	ephy_location_entry_autocompletion_hide_alternatives (EphyLocationEntry *w);
+static void	ephy_location_entry_autocompletion_window_hidden_cb
+							(EphyAutocompletionWindow *aw,
+							 EphyLocationEntry *w);
+static void
+insert_text_cb (GtkWidget *editable,
+		char *new_text,
+                int new_text_length,
+                int *position,
+		EphyLocationEntry *w);
+static void
+delete_text_cb (GtkWidget *editable,
+                int start_pos,
+                int end_pos,
+		EphyLocationEntry *w);
 
-
-
-
-static gpointer gtk_hbox_class;
+static GObjectClass *parent_class = NULL;
 
 /**
  * Signals enums and ids
@@ -96,23 +107,67 @@ static gpointer gtk_hbox_class;
 enum EphyLocationEntrySignalsEnum {
 	ACTIVATED,
 	FINISHED,
+	USER_CHANGED,
 	LAST_SIGNAL
 };
 static gint EphyLocationEntrySignals[LAST_SIGNAL];
 
-/**
- * EphyLocationEntry object
- */
+#define MENU_ID "ephy-location-entry-menu-id"
 
-MAKE_GET_TYPE (ephy_location_entry, "EphyLocationEntry", EphyLocationEntry,
-	       ephy_location_entry_class_init,
-	       ephy_location_entry_init, GTK_TYPE_HBOX);
+GType
+ephy_location_entry_get_type (void)
+{
+	static GType ephy_location_entry_type = 0;
+
+	if (ephy_location_entry_type == 0)
+	{
+		static const GTypeInfo our_info =
+		{
+			sizeof (EphyLocationEntryClass),
+			NULL,
+			NULL,
+			(GClassInitFunc) ephy_location_entry_class_init,
+			NULL,
+			NULL,
+			sizeof (EphyLocationEntry),
+			0,
+			(GInstanceInitFunc) ephy_location_entry_init
+		};
+
+		ephy_location_entry_type = g_type_register_static (EGG_TYPE_TOOL_ITEM,
+							           "EphyLocationEntry",
+							           &our_info, 0);
+	}
+
+	return ephy_location_entry_type;
+}
+
+static gboolean
+ephy_location_entry_set_tooltip (EggToolItem *tool_item,
+				 GtkTooltips *tooltips,
+				 const char *tip_text,
+				 const char *tip_private)
+{
+	EphyLocationEntry *entry = EPHY_LOCATION_ENTRY (tool_item);
+
+	g_return_val_if_fail (EPHY_IS_LOCATION_ENTRY (entry), FALSE);
+
+	gtk_tooltips_set_tip (tooltips, entry->priv->entry, tip_text, tip_private);
+
+	return TRUE;
+}
 
 static void
 ephy_location_entry_class_init (EphyLocationEntryClass *klass)
 {
-	G_OBJECT_CLASS (klass)->finalize = ephy_location_entry_finalize_impl;
-	gtk_hbox_class = g_type_class_peek_parent (klass);
+	GObjectClass *object_class = G_OBJECT_CLASS (klass);
+	EggToolItemClass *tool_item_class = EGG_TOOL_ITEM_CLASS (klass);
+
+	parent_class = g_type_class_peek_parent (klass);
+
+	object_class->finalize = ephy_location_entry_finalize_impl;
+
+	tool_item_class->set_tooltip = ephy_location_entry_set_tooltip;
 
 	EphyLocationEntrySignals[ACTIVATED] = g_signal_new (
 		"activated", G_OBJECT_CLASS_TYPE (klass),
@@ -133,13 +188,20 @@ ephy_location_entry_class_init (EphyLocationEntryClass *klass)
 		G_TYPE_NONE,
 		0,
 		G_TYPE_NONE);
+	EphyLocationEntrySignals[USER_CHANGED] = g_signal_new (
+		"user_changed", G_OBJECT_CLASS_TYPE (klass),
+		G_SIGNAL_RUN_FIRST | G_SIGNAL_RUN_LAST | G_SIGNAL_RUN_CLEANUP,
+                G_STRUCT_OFFSET (EphyLocationEntryClass, user_changed),
+		NULL, NULL,
+		g_cclosure_marshal_VOID__VOID,
+		G_TYPE_NONE,
+		0,
+		G_TYPE_NONE);
 }
 
 static gboolean
 location_focus_out_cb (GtkWidget *widget, GdkEventFocus *event, EphyLocationEntry *w)
 {
-	w->priv->editing = FALSE;
-
 	g_signal_emit (w, EphyLocationEntrySignals[FINISHED], 0);
 
 	return FALSE;
@@ -148,13 +210,21 @@ location_focus_out_cb (GtkWidget *widget, GdkEventFocus *event, EphyLocationEntr
 static void
 ephy_location_entry_init (EphyLocationEntry *w)
 {
-	EphyLocationEntryPrivate *p = g_new0 (EphyLocationEntryPrivate, 1);
+	EphyLocationEntryPrivate *p;
+
+	LOG ("EphyLocationEntry initialising %p", w)
+
+	p = g_new0 (EphyLocationEntryPrivate, 1);
 	w->priv = p;
 	p->last_action_target = NULL;
-	p->editing = FALSE;
 	p->before_completion = NULL;
+	p->user_changed = TRUE;
+	p->autocompletion_key = NULL;
+	p->autocompletion_window_visible = FALSE;
 
-	ephy_location_entry_build (w);
+	ephy_location_entry_construct_contents (w);
+
+	egg_tool_item_set_expand (EGG_TOOL_ITEM (w), TRUE);
 
 	g_signal_connect (w->priv->entry,
 			  "focus_out_event",
@@ -183,9 +253,10 @@ ephy_location_entry_finalize_impl (GObject *o)
 	LOG ("EphyLocationEntry finalized")
 
 	g_free (p->before_completion);
+	g_free (p->autocompletion_key);
 
 	g_free (p);
-	G_OBJECT_CLASS (gtk_hbox_class)->finalize (o);
+	G_OBJECT_CLASS (parent_class)->finalize (o);
 }
 
 GtkWidget *
@@ -194,19 +265,43 @@ ephy_location_entry_new (void)
 	return GTK_WIDGET (g_object_new (EPHY_TYPE_LOCATION_ENTRY, NULL));
 }
 
+static gboolean
+ephy_location_entry_button_press_event_cb (GtkWidget *entry, GdkEventButton *event, EphyLocationEntry *w)
+{
+	if (event->button == 1 && event->type == GDK_2BUTTON_PRESS)
+	{
+		ephy_location_entry_activate (w);
+		return TRUE;
+	}
+
+	return FALSE;
+}
+
 static void
-ephy_location_entry_build (EphyLocationEntry *w)
+ephy_location_entry_construct_contents (EphyLocationEntry *w)
 {
 	EphyLocationEntryPrivate *p = w->priv;
-	GtkWidget *list;
+	GtkWidget *hbox, *list;
+
+	LOG ("EphyLocationEntry constructing contents %p", w)
+
+	hbox = gtk_hbox_new (FALSE, 0);
+	gtk_container_add (GTK_CONTAINER (w), hbox);
+	gtk_widget_show (hbox);
 
 	p->combo = gnome_entry_new ("ephy-url-history");
 	p->entry = GTK_COMBO (p->combo)->entry;
 	gtk_widget_show (p->combo);
-	gtk_box_pack_start (GTK_BOX (w), p->combo, TRUE, TRUE, 0);
+	gtk_box_pack_start (GTK_BOX (hbox), p->combo, TRUE, TRUE, 0);
 
 	g_signal_connect (p->entry, "key-press-event",
 			  G_CALLBACK (ephy_location_entry_key_press_event_cb), w);
+	g_signal_connect (p->entry, "insert-text",
+			  G_CALLBACK (insert_text_cb), w);
+	g_signal_connect (p->entry, "delete-text",
+			  G_CALLBACK (delete_text_cb), w);
+	g_signal_connect (p->entry, "button-press-event",
+			  G_CALLBACK (ephy_location_entry_button_press_event_cb), w);
 
 	g_signal_connect (p->entry, "activate",
 			  G_CALLBACK (ephy_location_entry_activate_cb), w);
@@ -260,8 +355,8 @@ ephy_location_entry_autocompletion_show_alternatives_to (EphyLocationEntry *w)
 {
 	EphyLocationEntryPrivate *p = w->priv;
 
-	g_free (p->before_completion),
-	p->before_completion = gtk_editable_get_chars (GTK_EDITABLE(p->entry), 0, -1);
+	g_free (p->before_completion);
+	p->before_completion = gtk_editable_get_chars (GTK_EDITABLE (p->entry), 0, -1);
 
 	if (ephy_location_ignore_prefix (w)) return FALSE;
 
@@ -284,12 +379,27 @@ ephy_location_entry_autocompletion_hide_alternatives (EphyLocationEntry *w)
 		ephy_autocompletion_window_hide (p->autocompletion_window);
 		p->autocompletion_window_visible = FALSE;
 	}
+
+	p->autocompletion_window_visible = FALSE;
+
+	if (p->show_alternatives_timeout)
+	{
+		g_source_remove (p->show_alternatives_timeout);
+		p->show_alternatives_timeout = 0;
+	}
 }
 
 static void
 ephy_location_entry_autocompletion_show_alternatives (EphyLocationEntry *w)
 {
 	EphyLocationEntryPrivate *p = w->priv;
+
+	/* Reset it because if we do a grab on click (like when pasting
+	   text in the location entry, the entry will lose the release
+	   event and will not reset it. It's what gtk does for the entry
+	   popup */
+	GTK_ENTRY (p->entry)->button = 0;
+
 	if (p->autocompletion_window)
 	{
 		ephy_autocompletion_window_show (p->autocompletion_window);
@@ -329,29 +439,29 @@ position_is_at_end (GtkEditable *editable)
 }
 
 static void
-real_entry_set_location (EphyLocationEntry *w,
-		         const gchar *new_location)
+delete_text_cb (GtkWidget *editable,
+                int start_pos,
+                int end_pos,
+		EphyLocationEntry *w)
 {
-	EphyLocationEntryPrivate *p = w->priv;
-	int pos;
-
-	gtk_editable_delete_text (GTK_EDITABLE (p->entry), 0, -1);
-	gtk_editable_insert_text (GTK_EDITABLE (p->entry), new_location, g_utf8_strlen (new_location, -1),
-				  &pos);
+	ephy_location_entry_autocompletion_hide_alternatives (w);
 }
 
-/* this is from the old location entry, need to do the autocompletion before implementing this */
-static gboolean
-ephy_location_entry_key_press_event_cb (GtkWidget *entry, GdkEventKey *event, EphyLocationEntry *w)
+static void
+insert_text_cb (GtkWidget *editable,
+		char *new_text,
+                int new_text_length,
+                int *position,
+		EphyLocationEntry *w)
 {
 	EphyLocationEntryPrivate *p = w->priv;
-        guint keyval = event->keyval;
+	GtkWidget *window;
 
-        if (p->autocompletion_timeout != 0)
-	{
-                g_source_remove (p->autocompletion_timeout);
-		p->autocompletion_timeout = 0;
-	}
+	window = gtk_widget_get_toplevel (editable);
+	g_return_if_fail (window != NULL);
+	if (!GTK_WINDOW (window)->has_focus) return;
+
+	if (p->going_to_site) return;
 
         if (p->show_alternatives_timeout != 0)
 	{
@@ -359,62 +469,42 @@ ephy_location_entry_key_press_event_cb (GtkWidget *entry, GdkEventKey *event, Ep
 		p->show_alternatives_timeout = 0;
 	}
 
-        if (((event->state & GDK_Control_L || event->state & GDK_Control_R) &&
-             (keyval == GDK_a || keyval == GDK_b || keyval == GDK_c ||
-              keyval == GDK_d || keyval == GDK_e || keyval == GDK_f ||
-              keyval == GDK_h || keyval == GDK_k || keyval == GDK_u ||
-              keyval == GDK_v || keyval == GDK_w || keyval == GDK_x)) ||
-            (event->state == 0 && event->keyval == GDK_BackSpace))
-        {
-		ephy_location_entry_autocompletion_hide_alternatives (w);
-                return FALSE;
-        }
+	ephy_location_entry_autocompletion_unselect_alternatives (w);
+	if (position_is_at_end (GTK_EDITABLE (editable)))
+	{
+		p->show_alternatives_timeout = g_timeout_add
+			(SHOW_ALTERNATIVES_DELAY,
+			 (GSourceFunc) ephy_location_entry_autocompletion_show_alternatives_to, w);
+	}
+}
 
-        /* don't grab alt combos, thus you can still access the menus. */
-        if (event->state & GDK_MOD1_MASK)
-        {
-		ephy_location_entry_autocompletion_hide_alternatives (w);
-                return FALSE;
-        }
-
-        /* make sure the end key works at all times */
-        if ((!((event->state & GDK_SHIFT_MASK) ||
-	       (event->state & GDK_CONTROL_MASK) ||
-	       (event->state & GDK_MOD1_MASK))
-	     && (event->keyval == GDK_End)))
-        {
-		ephy_location_entry_autocompletion_hide_alternatives (w);
-                gtk_editable_select_region (GTK_EDITABLE (p->entry), 0, 0);
-                gtk_editable_set_position (GTK_EDITABLE (p->entry), -1);
-		ephy_location_entry_autocompletion_unselect_alternatives (w);
-                return TRUE;
-        }
+static gboolean
+ephy_location_entry_key_press_event_cb (GtkWidget *entry, GdkEventKey *event, EphyLocationEntry *w)
+{
+	EphyLocationEntryPrivate *p = w->priv;
+	GtkWidget *window;
 
 	switch (event->keyval)
         {
         case GDK_Left:
         case GDK_Right:
-        case GDK_Up:
-        case GDK_Down:
-        case GDK_Page_Up:
-        case GDK_Page_Down:
 		ephy_location_entry_autocompletion_hide_alternatives (w);
-                return FALSE;
+                break;
 	case GDK_Escape:
-		real_entry_set_location (w, p->before_completion);
+		ephy_location_entry_set_location (w, p->before_completion);
 		gtk_editable_set_position (GTK_EDITABLE (p->entry), -1);
 		ephy_location_entry_autocompletion_hide_alternatives (w);
-                return FALSE;
+                break;
+	case GDK_Tab:
+	case GDK_KP_Tab:
+		if (p->autocompletion_window_visible)
+		{
+			ephy_location_entry_autocompletion_hide_alternatives (w);
+			window = gtk_widget_get_toplevel (entry);
+			g_signal_emit_by_name (window, "move_focus", 1);
+		}
+		break;
         default:
-		w->priv->editing = TRUE;
-		ephy_location_entry_autocompletion_unselect_alternatives (w);
-		if ((event->string[0] > 32) && (event->string[0] < 126) &&
-		    position_is_at_end (GTK_EDITABLE (entry)))
-                {
-                        p->show_alternatives_timeout = g_timeout_add
-                                (SHOW_ALTERNATIVES_DELAY,
-				 (GSourceFunc) ephy_location_entry_autocompletion_show_alternatives_to, w);
-                }
                 break;
         }
 
@@ -425,7 +515,8 @@ static gboolean
 ephy_location_entry_content_is_text (const char *content)
 {
 	return ((g_strrstr (content, ".") == NULL) &&
-		(g_strrstr (content, "/") == NULL));
+		(g_strrstr (content, "/") == NULL) &&
+		(g_strrstr (content, ":") == NULL));
 }
 
 static void
@@ -435,21 +526,28 @@ ephy_location_entry_activate_cb (GtkEntry *entry, EphyLocationEntry *w)
 	char *target = NULL;
 
 	content = gtk_editable_get_chars (GTK_EDITABLE(entry), 0, -1);
-	if (ephy_location_entry_content_is_text (content))
+	if (w->priv->last_action_target &&
+	    ephy_location_entry_content_is_text (content))
 	{
-		target = w->priv->last_action_target;
+		target = g_strdup (w->priv->last_action_target);
+	}
+	else
+	{
+		target = content;
+		content = NULL;
+		g_free ( w->priv->last_action_target);
+		w->priv->last_action_target = NULL;
 	}
 
 	ephy_location_entry_autocompletion_hide_alternatives (w);
 
 	LOG ("In ephy_location_entry_activate_cb, activating %s", content)
 
-	w->priv->editing = FALSE;
-
-	g_signal_emit (w, EphyLocationEntrySignals[ACTIVATED], 0, target, content);
+	g_signal_emit (w, EphyLocationEntrySignals[ACTIVATED], 0, content, target);
 	g_signal_emit (w, EphyLocationEntrySignals[FINISHED], 0);
 
 	g_free (content);
+	g_free (target);
 }
 
 static void
@@ -473,10 +571,25 @@ void
 ephy_location_entry_set_location (EphyLocationEntry *w,
 				  const gchar *new_location)
 {
-	if (!w->priv->editing)
-	{
-		real_entry_set_location (w, new_location);
-	}
+	EphyLocationEntryPrivate *p = w->priv;
+	int pos;
+
+	p->user_changed = FALSE;
+
+	g_signal_handlers_block_by_func (G_OBJECT (p->entry),
+				         delete_text_cb, w);
+	gtk_editable_delete_text (GTK_EDITABLE (p->entry), 0, -1);
+	g_signal_handlers_unblock_by_func (G_OBJECT (p->entry),
+				           delete_text_cb, w);
+
+	g_signal_handlers_block_by_func (G_OBJECT (p->entry),
+				         insert_text_cb, w);
+	gtk_editable_insert_text (GTK_EDITABLE (p->entry), new_location, strlen(new_location),
+				  &pos);
+	g_signal_handlers_unblock_by_func (G_OBJECT (p->entry),
+				           insert_text_cb, w);
+
+	p->user_changed = TRUE;
 }
 
 gchar *
@@ -494,13 +607,14 @@ ephy_location_entry_autocompletion_window_url_selected_cb (EphyAutocompletionWin
 {
 	if (target)
 	{
-		real_entry_set_location (w, action ? w->priv->before_completion : target);
+		ephy_location_entry_set_location (w, action ? w->priv->before_completion : target);
 	}
 	else
 	{
-		real_entry_set_location (w, w->priv->before_completion);
-		gtk_editable_set_position (GTK_EDITABLE (w->priv->entry), -1);
+		ephy_location_entry_set_location (w, w->priv->before_completion);
 	}
+
+	gtk_editable_set_position (GTK_EDITABLE (w->priv->entry), -1);
 }
 
 void
@@ -562,7 +676,7 @@ ephy_location_entry_autocompletion_window_url_activated_cb (EphyAutocompletionWi
 	}
 	else
 	{
-		real_entry_set_location (w, target);
+		ephy_location_entry_set_location (w, target);
 	}
 
 	content = gtk_editable_get_chars (GTK_EDITABLE(w->priv->entry), 0, -1);
@@ -592,25 +706,6 @@ ephy_location_entry_autocompletion_window_hidden_cb (EphyAutocompletionWindow *a
 		g_source_remove (p->show_alternatives_timeout);
 		p->show_alternatives_timeout = 0;
 	}
-
-	if (p->autocompletion_timeout)
-	{
-		g_source_remove (p->autocompletion_timeout);
-		p->autocompletion_timeout = 0;
-	}
-}
-
-void
-ephy_location_entry_edit (EphyLocationEntry *w)
-{
-	GtkWidget *toplevel;
-
-	w->priv->editing = TRUE;
-
-	toplevel = gtk_widget_get_toplevel (w->priv->entry);
-
-        gtk_window_set_focus (GTK_WINDOW(toplevel),
-                              w->priv->entry);
 }
 
 void
@@ -653,8 +748,13 @@ ephy_location_entry_editable_changed_cb (GtkEditable *editable, EphyLocationEntr
 		{
 			p->going_to_site = FALSE;
 			g_signal_emit (e, EphyLocationEntrySignals[ACTIVATED], 0, NULL, url);
-			g_free (url);
 		}
+		g_free (url);
+	}
+
+	if (p->user_changed)
+	{
+		g_signal_emit (e, EphyLocationEntrySignals[USER_CHANGED], 0);
 	}
 }
 
